from django.db import models
from django.contrib.auth.models import AbstractUser, Group, Permission
from django import forms
import datetime
# Create your models here.


class User(AbstractUser):
    date_of_birth = models.DateField(default=datetime.date.today)
    is_rep=models.BooleanField(default=False)

    
class AccountManager(User):
    pass

class CinemaManager(User):
    cinema = models.ForeignKey(
        'Cinema',
        on_delete=models.CASCADE,
        related_name='cinema',
        null=True,
        blank=True
    )

class Club(models.Model):
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=50)
    representative = models.ForeignKey(
        'User',
        on_delete=models.CASCADE,
        related_name='clubs',
        null=True,
        blank=True
    )
    address_details = models.TextField(max_length=100, default='')
    contact_details = models.TextField(max_length=50, default='')
    account = models.ForeignKey(
        'Account',
        on_delete=models.CASCADE,
        related_name='club_account',
        null=True,
        blank=True
    )

class PaymentDetails(models.Model):
    details_name = models.CharField(max_length=32)
    payment_card_number = models.CharField(max_length=20)
    payment_card_expiry_date = models.DateField()

    
class Account(models.Model):
    id = models.AutoField(primary_key=True)
    payment_card_number = models.CharField(max_length=20)
    payment_card_expiry_date = models.DateField()
    discount_rate = models.DecimalField(max_digits=5, decimal_places=2)
    end_of_month_statements = models.TextField()
    
    def create_statement(account_id):
        account = Account.objects.get(pk=account_id)
        receipts = PaymentReceipt.objects.filter(account__pk=account_id)

        total_spent = 0
        total_paid = 0
        for receipt in receipts:
            total_spent += receipt.required_amount
            total_paid += receipt.paid_amount
            
        outstanding = total_spent - total_paid
        
        return EndOfMonthStatement.objects.create(
            account=account,
            total_spent=total_spent,
            total_paid=total_paid,
            outstanding=outstanding
        )


class Cinema(models.Model):
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=255)
    location = models.CharField(max_length=255)

class Film(models.Model):
    id = models.AutoField(primary_key=True)
    title = models.CharField(max_length=255)
    length = models.PositiveIntegerField()
    rating = models.PositiveSmallIntegerField()
    genre = models.CharField(max_length=50)

class Screen(models.Model):
    id = models.AutoField(primary_key=True)
    cinema = models.ForeignKey(Cinema, on_delete=models.CASCADE, related_name='screens')
    screen_number = models.PositiveSmallIntegerField()
    seating_capacity = models.PositiveIntegerField()\

class Showing(models.Model):
    id = models.AutoField(primary_key=True)
    film = models.ForeignKey(Film, on_delete=models.CASCADE, related_name='showings')
    screen = models.ForeignKey(Screen, on_delete=models.CASCADE, related_name='showings')
    start_time = models.DateTimeField()


class ClubRepresentative(User):
    representative_id = models.CharField(max_length=50, unique=True)
    
    USERNAME_FIELD='representative_id'
    REQUIRED_FIELDS = ['password']


class EndOfMonthStatement(models.Model):
    account = models.ForeignKey(Account, on_delete=models.CASCADE, related_name="eom_account")
    date = models.DateField(default=datetime.date.today)
    total_spent = models.DecimalField(max_digits = 10, decimal_places=2)
    total_paid = models.DecimalField(max_digits = 10, decimal_places=2)
    outstanding = models.DecimalField(max_digits = 10, decimal_places=2)
    
    def update_outstanding(statement_id):
        statement = EndOfMonthStatement.objects.get(pk=statement_id)
        return statement.total_spent - statement.total_paid

class Seat(models.Model):
    screen = models.ForeignKey(Screen, on_delete=models.CASCADE, related_name="seats")
    


class PaymentReceipt(models.Model):
    account = models.ForeignKey(Account, on_delete=models.CASCADE, related_name="account_receipt")
    payment_user = models.ForeignKey(User, on_delete=models.CASCADE, related_name="user_of_payment")
    payment_card_details = models.ForeignKey(PaymentDetails, on_delete=models.CASCADE, related_name="payment_card_details")
    required_amount = models.DecimalField(max_digits = 10, decimal_places=2)
    paid_amount = models.DecimalField(max_digits=10, decimal_places=2)